@@ -8,7 +8,7 @@ from functools import wraps
 
 
 __all__ = ['Dispatcher', 'command', 'dispatch']
-__version__ = '3.2.2'
+__version__ = '3.3'
 __author__ = 'Alexander Solovyov'
 __email__ = 'alexander@solovyov.net'
 
@@ -358,7 +358,7 @@ def help_options(options):
 # Options process
 # --------
 
-def process(args, options):
+def process(args, options, preparse=False):
     '''
     >>> opts = [('l', 'listen', 'localhost',
     ...          'ip to listen on'),
@@ -409,7 +409,16 @@ def process(args, options):
         if name:
             namelist.append(name)
 
-    opts, args = getopt.gnu_getopt(args, shortlist, namelist)
+    try:
+        opts, args = getopt.gnu_getopt(args, shortlist, namelist)
+    except getopt.GetoptError, e:
+        if preparse:
+            prefix = '-' if len(e.opt) == 1 else '--'
+            args = args[:]
+            args.insert(args.index(prefix + e.opt), '--')
+            opts, args = getopt.gnu_getopt(args, shortlist, namelist)
+            return args, None
+        raise
 
     # transfer result to state
     for opt, val in opts:
@@ -446,12 +455,10 @@ def process(args, options):
 def cmdparse(args, cmdtable, globalopts):
     '''Parse arguments list to find a command, options and arguments
     '''
-    # command is the first non-option
-    cmd = None
-    for arg in args:
-        if not arg.startswith('-'):
-            cmd = arg
-            break
+    # pre-parse arguments here using global options to find command name,
+    # which is first non-option entry
+    cmd = next((arg for arg in process(args, globalopts, preparse=True)[0]
+                if not arg.startswith('-')), None)
 
     if cmd:
         args.pop(args.index(cmd))
